@@ -7,7 +7,9 @@ import io
 import json
 import os
 import re
+import requests
 import sys
+import time
 from asyncio import AbstractEventLoop
 from datetime import datetime
 from threading import Thread, Lock
@@ -31,11 +33,13 @@ ingested_bytes = Counter('ingested_bytes', 'Total number of bytes ingested', ['r
 ingested_documents = Counter('ingested_documents', 'Total number of documents ingested', ['region', 'tenant', 'fabric', 'workflow'])
 ingest_errors = Counter('ingest_errors', 'Total number of errors during ingestion', ['region', 'tenant', 'fabric', 'workflow'])
 ingest_lag = Gauge('ingest_lag', 'Average time lag between data changes in GDN collections and external data sources', ['region', 'tenant', 'fabric', 'workflow'])
+scrape_complete_flag = Counter("scrape_complete_flag", "Flag to check if the last scrape has been completed", ['workflow'])
 
 region_label = os.getenv("GDN_FEDERATION", "NA")
 tenant_label = os.getenv("GDN_TENANT", "NA")
 fabric_label = os.getenv("GDN_FABRIC", "NA")
 workflow_label = os.getenv("WORKFLOW_UUID", "NA")
+metric_service_url = os.getenv("METRIC_SERVICE_API")
 
 # Start the Prometheus HTTP server for exposing metrics
 start_http_server(8000)
@@ -89,6 +93,8 @@ def try_upsert(collection: StandardCollection, record_batch: RecordBatch, force=
                 # Update ingested_documents and ingested_bytes metrics
                 ingested_documents.labels(region_label, tenant_label, fabric_label, workflow_label).inc()
                 ingested_bytes.labels(region_label, tenant_label, fabric_label, workflow_label).inc(len(json.dumps(r)))
+                # Update ingest_lag metric
+                ingest_lag.labels(region_label, tenant_label, fabric_label, workflow_label).set((datetime.now() - to_insert[i].time_extracted).total_seconds())
 
         if len(to_update) > 0:
             for i, r in enumerate(collection.update_many(to_update)):
@@ -96,8 +102,9 @@ def try_upsert(collection: StandardCollection, record_batch: RecordBatch, force=
                     # Increment ingest_errors metric
                     ingest_errors.labels(region_label, tenant_label, fabric_label, workflow_label).inc()
                     print(f'Failed to insert/update record: {to_update[i]}. {r}')
-        # Update ingest_lag metric
-        ingest_lag.labels(region_label, tenant_label, fabric_label, workflow_label).set((datetime.now() - record_batch.last_executed_time).total_seconds())
+                else:
+                    # Update ingest_lag metric
+                    ingest_lag.labels(region_label, tenant_label, fabric_label, workflow_label).set((datetime.now() - to_update[i].time_extracted).total_seconds())
         record_batch.last_executed_time = datetime.now()
 
 
@@ -143,6 +150,10 @@ def persist_messages(collection: StandardCollection, messages: io.TextIOWrapper,
 
             try:
                 rec = o['record']
+                if o["time_extracted"]:
+                    rec["time_extracted"] = o["time_extracted"]
+                else:
+                    rec["time_extracted"] = record_batch.last_executed_time
                 try:
                     kps = key_properties.get(stream)
                     _key = None
@@ -209,6 +220,7 @@ def persist_messages(collection: StandardCollection, messages: io.TextIOWrapper,
             ingest_errors.labels(region_label, tenant_label, fabric_label, workflow_label).inc()
             logger.warning("Unknown message type {} in message {}".format(o['type'], o))
 
+    scrape_complete_flag.labels(workflow_label).inc()
     return state
 
 
@@ -253,13 +265,16 @@ def main():
         apikey=apikey,
         geofabric=fabric
     )
+
     if not client.has_collection(target_collection):
         client.create_collection(name=target_collection)
+
     collection = client.get_collection(target_collection)
     record_batch = RecordBatch(config)
     event_loop = setup_batch_task(collection, record_batch)
     input_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
     state = persist_messages(collection, input_messages, record_batch)
+
     # There can still be records in the `record_batch` which is not processed,
     # So, we have to force process it one last time before the workflow terminates.
     try_upsert(collection, record_batch, force=True)
@@ -268,5 +283,18 @@ def main():
     logger.info("Exiting normally...")
 
 
+def is_scrape_complete(prometheus_url, metric_name, filter):
+    response = requests.get(f"{prometheus_url}/query", params={"query": f"{metric_name}{{{filter}}}"})
+    if response.status_code == 200:
+        json_data = response.json()
+        if json_data["data"]["result"] and len(json_data["data"]["result"]) > 0:
+            return True
+    return False
+
+
 if __name__ == '__main__':
     main()
+
+    # Wait for Prometheus to scrape the metrics
+    while not is_scrape_complete(metric_service_url, f"{scrape_complete_flag._name}_total", f"workflow=\"{workflow_label}\""):
+        time.sleep(15)
